﻿// // ******************************************************************
// //       /\ /|       @File         ProtocolBase.cs
// //       \ V/        @Brief
// //       | "")       @Author        lijinwen, ghz005@uni-trend.com.cn
// //       /  |        @Creation      2024-1-22
// //      /  \\        @Modified      2024-1-26
// //    *(__\_\
// // ******************************************************************
using System.Diagnostics;
using System.Runtime.InteropServices;

namespace ComModel;

public class ProtocolBase
{
    public enum TwoLevelEdgePulseStatusType
    {
        Low,
        High,
        None
    };
    public enum OddEvenCheck
    {
        None = 0,
        Even = 2,
        Odd = 1
    }
    [StructLayout(LayoutKind.Sequential)]
    public struct EdgePulseData
    {
        public UInt64 WaveformDataCount;
        public UInt64 EdgePulsesCount;
        public double SampleRate;
        public CommonBase.ChannelId ChannelId;
        IntPtr edgePulsesPtr;

        public EdgePulseData(EdgePulse[] edgePulses, UInt64 edgePulsesCount, UInt64 waveformDataCount
            , CommonBase.ChannelId channelId, double sampleRate)
        {
            WaveformDataCount = waveformDataCount;
            edgePulsesPtr = Marshal.UnsafeAddrOfPinnedArrayElement(edgePulses, 0);
            ChannelId = channelId;
            SampleRate = sampleRate;
            EdgePulsesCount = edgePulsesCount;
        }
         
    }
    [StructLayout(LayoutKind.Sequential)]
    public struct TwoLevelEdgePulseData
    {
        public UInt64 WaveformDataCount;
        public UInt64 EdgePulsesCount;
        public double SampleRateByHz;
        //public CommonBase.ChannelId ChannelId;
        IntPtr edgePulsesPtr;

        public TwoLevelEdgePulseData(TwoLevelEdgePulse[] edgePulses, UInt64 edgePulsesCount, UInt64 waveformDataCount
            , double sampleRateByHz)
        {
            WaveformDataCount = waveformDataCount;
            edgePulsesPtr = Marshal.UnsafeAddrOfPinnedArrayElement(edgePulses, 0);
            Console.WriteLine($"TwoLevelEdgePulseData edgePulsesPtr:{edgePulsesPtr:X}");
            //ChannelId = channelId;
            SampleRateByHz = sampleRateByHz;
            EdgePulsesCount = edgePulsesCount;
        }
         
    }
    //todo
    // [StructLayout(LayoutKind.Sequential)]
    // public struct ThreeLevelEdgePulseData
    // {
    //     public UInt64 WaveformDataCount;
    //     public UInt64 EdgePulsesCount;
    //     public double SampleRate;
    //     public CommonBase.ChannelId ChannelId;
    //     IntPtr edgePulsesPtr;
    //
    //     public ThreeLevelEdgePulseData(ThreeLevelEdgePulse[] edgePulses, UInt64 edgePulsesCount, UInt64 waveformDataCount
    //         , CommonBase.ChannelId channelId, double sampleRate)
    //     {
    //         WaveformDataCount = waveformDataCount;
    //         edgePulsesPtr = Marshal.UnsafeAddrOfPinnedArrayElement(edgePulses, 0);
    //         ChannelId = channelId;
    //         SampleRate = sampleRate;
    //         EdgePulsesCount = edgePulsesCount;
    //     }
    //      
    // }
    [StructLayout(LayoutKind.Sequential)]
    public struct EdgePulse
    {
        public uint LevelCount() { return 0; }
        public Int32 StartIndex;
        public Int32 EndIndex;
        public CommonBase.Edge Edge;
        public EdgePulse(int startIndex, int endIndex, CommonBase.Edge edge)
        {
            StartIndex = startIndex;
            EndIndex = endIndex;
            Edge = edge;
        }
    };

    [StructLayout(LayoutKind.Sequential)]
    public struct TwoLevelEdgePulse
    {
        public uint LevelCount() { return 2; }
        public Int32 StartIndex;
        public Int32 EndIndex;
        public CommonBase.Edge Edge;
        public TwoLevelEdgePulseStatusType CurrentLevel = TwoLevelEdgePulseStatusType.None;
        public TwoLevelEdgePulse(int startIndex, int endIndex, CommonBase.Edge edge)
        {
            StartIndex = startIndex;
            EndIndex = endIndex;
            Edge = edge;
            if (Edge == CommonBase.Edge.None)
            {
                return;
            }
            CurrentLevel = Edge == CommonBase.Edge.Rise ? TwoLevelEdgePulseStatusType.High : TwoLevelEdgePulseStatusType.Low;
        }

        public TwoLevelEdgePulse(EdgePulse edgePulse)
        {
            StartIndex = edgePulse.StartIndex;
            EndIndex = edgePulse.EndIndex;
            Edge = edgePulse.Edge;
            if (Edge == CommonBase.Edge.None)
            {
                return;
            }
            CurrentLevel = Edge == CommonBase.Edge.Rise ? TwoLevelEdgePulseStatusType.High : TwoLevelEdgePulseStatusType.Low;

        }

    }

}
