﻿// // ******************************************************************
// //       /\ /|       @File         struct RS232DecodeResult.cs
// //       \ V/        @Brief
// //       | "")       @Author        lijinwen, ghz005@uni-trend.com.cn
// //       /  |        @Creation      2024-1-22
// //      /  \\        @Modified      2024-1-26
// //    *(__\_\
// // ******************************************************************

//using System.Drawing;
using System.Runtime.InteropServices;
using ComModel;

namespace TekWaveformSampling.Model;

public class RS232DecodeResultUnit
{
    public long StartIndex;
    public long Length;
    public readonly List<byte> Datas;

    public bool IsEventInfo;
    public RS232DecodeResultUnit()
    {
        StartIndex = -1;
        Length = 0;
        Datas = new List<byte>();
    }
    internal RS232DecodeResultUnit(RS232DecodeResultUnitStruct decodeResult)
    {
        StartIndex = decodeResult.StartIndex;
        Length = decodeResult.Length;
        Datas = new List<byte>();
        IsEventInfo = decodeResult.IsEventInfo;
        //232 只有一个字节数据
        if (Length > 0)
        {
            IntPtr ptr = decodeResult.Datas;
            Datas.Add((byte)(Marshal.PtrToStructure(ptr, typeof(byte)) ?? throw new InvalidOperationException()));

        }
    }
}
public class RS232DecodeEventUint
{
    public long StartIndex;
    public long Length;
    public byte Data;

    public Enums.RS232DecodeEventType EventType;
    internal RS232DecodeEventUint(RS232DecodeEventUintStruct uintStruct)
    {
        StartIndex = uintStruct.StartIndex;
        Length = uintStruct.Length;
        Data = uintStruct.Data;
        EventType = uintStruct.EventType;
    }
}
public class RS232DecodeEvent
{
    public long EventIndex;
    public long StartIndex;
    public long EndIndex;
    public string Name = "";
    public byte ParityResult; //string ptr  
    public List<RS232DecodeEventUint> EventDatas;
    internal RS232DecodeEvent(RS232DecodeEventStruct decodeStruct)
    {
        EventIndex = decodeStruct.EventIndex;
        StartIndex = decodeStruct.StartIndex;
        EndIndex = decodeStruct.EventIndex;

        ParityResult = decodeStruct.ParityResult;
        EventDatas = new List<RS232DecodeEventUint>();
        IntPtr ptr = decodeStruct.EventDatas;
        //results = new RS232DecodeResultUnitStruct[dataStruct.DecodeResultCount];
        // 计算结构体的字节数
        int structSize = Marshal.SizeOf(typeof(RS232DecodeEventUintStruct));
        if (decodeStruct.EventDatasCount > 0)
        {
            for (int i = 0; i < decodeStruct.EventDatasCount; i++)
            {
                IntPtr structPtr = new(ptr.ToInt64() + ((long)i * structSize));
                EventDatas.Add(new RS232DecodeEventUint((RS232DecodeEventUintStruct)(Marshal.PtrToStructure(structPtr, typeof(RS232DecodeEventUintStruct)) ?? throw new InvalidOperationException())));
            }
        }
    }
}
public class RS232DecodeResult
{
    #region 属性

    public bool ResultValid;
    public bool DecodeResultNeedUpdate;
    public bool DecodeEventNeedUpdate;
    public List<RS232DecodeResultUnit> DecodeResultUnits;
    public List<RS232DecodeEvent> DecodeEvents;
    public RS232DecodeResult()
    {
        DecodeResultUnits = new List<RS232DecodeResultUnit>();
        DecodeEvents = new List<RS232DecodeEvent>();
    }
    public int DecodeResultCount
    {
        get
        {
            return DecodeResultUnits.Count;
        }
    }
    public int DecodeEventCount
    {
        get
        {
            return DecodeEvents.Count;
        }
    }

    #endregion 属性

    #region 方法

    public CommonBase.SerialProtocolType GetProtocolType()
    {
        return CommonBase.SerialProtocolType.RS232;
    }
    public static bool ConvertData(RS232DecodeResultStruct dataStruct, out RS232DecodeResult dataResult)
    {

        dataResult = new RS232DecodeResult();

        if (dataStruct.DecodeEventCount > 0)
        {
            IntPtr ptr = dataStruct.DecodeEvents;
            //events = new RS232DecodeEventStruct[dataStruct.DecodeEventCount];
            // 计算结构体的字节数
            int structSize = Marshal.SizeOf(typeof(RS232DecodeEventStruct));
            for (ulong i = 0; i < dataStruct.DecodeEventCount; i++)
            {
                IntPtr structPtr = new(ptr.ToInt64() + ((long)i * structSize));
                RS232DecodeEventStruct decodeEventStruct = (RS232DecodeEventStruct)(Marshal.PtrToStructure(structPtr, typeof(RS232DecodeEventStruct)) ?? throw new InvalidOperationException());
                RS232DecodeEvent decodeEvent = new(decodeEventStruct);
                dataResult.DecodeEvents.Add(decodeEvent);
            }
            //test out 
            Console.WriteLine(@"    First Event: ");
            Console.WriteLine($@"        StartIndex:{dataResult.DecodeEvents[0].StartIndex} ");

        }

        if (dataStruct.DecodeResultCount > 0)
        {
            IntPtr ptr = dataStruct.DecodeResultUnits;
            Console.WriteLine($@"DecodeResultUnits Ptr:0x{ptr:X}");
            //results = new RS232DecodeResultUnitStruct[dataStruct.DecodeResultCount];
            // 计算结构体的字节数
            int structSize = Marshal.SizeOf(typeof(RS232DecodeResultUnitStruct));
            for (ulong i = 0; i < dataStruct.DecodeResultCount; i++)
            {
                IntPtr structPtr = new(ptr.ToInt64() + ((long)i * structSize));
                RS232DecodeResultUnitStruct rS232DecodeResultUnitStruct = (RS232DecodeResultUnitStruct)(Marshal.PtrToStructure(structPtr, typeof(RS232DecodeResultUnitStruct)) ?? throw new InvalidOperationException());
                RS232DecodeResultUnit rS232DecodeResultUnit = new(rS232DecodeResultUnitStruct);
                dataResult.DecodeResultUnits.Add(rS232DecodeResultUnit);
            }
            //test out 
            Console.WriteLine(@"    First Data: ");
            Console.WriteLine($@"        StartIndex:{dataResult.DecodeResultUnits[0].StartIndex} ");
        }
        return true;
    }

    #endregion 方法


}
