﻿// // ******************************************************************
// //       /\ /|       @File         WaveFormSampling.cs
// //       \ V/        @Brief
// //       | "")       @Author        lijinwen, ghz005@uni-trend.com.cn
// //       /  |        @Creation      2024-1-22
// //      /  \\        @Modified      2024-1-26
// //    *(__\_\
// // ******************************************************************
using ComModel;
using static ComModel.ProtocolBase;

namespace TekWaveformSampling;

public class WaveFormSampling
{
    public static bool Sampling(TekWaveformData? waveformData, double thresholdHigh, double thresholdLow, out List<ProtocolBase.TwoLevelEdgePulse> edgePulses)
    {
        edgePulses = new List<ProtocolBase.TwoLevelEdgePulse>();
        if (waveformData == null || waveformData.SampleInterval == 0 || waveformData.WaveformDatas == null || !waveformData.WaveformDatas.Any())
        {
            return false;
        }
        CommonBase.Edge nowEdge = CommonBase.Edge.None;
        ProtocolBase.EdgePulse? lastEdgePulse = null;

        for (int i = 0; i < waveformData.WaveformDatas.Count; i++)
        {
            WaveformData? data = waveformData.WaveformDatas[i];
            if (data.Voltage >= thresholdLow && data.Voltage <= thresholdHigh && edgePulses.Count() > 0)
            {
                //延申
                //var changeEdgePulse = edgePulses[edgePulses.Count - 1];
                //changeEdgePulse.EndIndex = i;
                //edgePulses[edgePulses.Count - 1] = changeEdgePulse;
                continue;
            }
            nowEdge = data.Voltage > thresholdHigh ? CommonBase.Edge.Rise : CommonBase.Edge.Falling;
            if (i == 0)
            {
                lastEdgePulse = new ProtocolBase.EdgePulse(0, 0, nowEdge);
                continue;
            }

            if (lastEdgePulse != null && nowEdge != lastEdgePulse.Value.Edge)
            {
                lastEdgePulse = lastEdgePulse.Value with { EndIndex = i };
             
                edgePulses.Add(new ProtocolBase.TwoLevelEdgePulse((ProtocolBase.EdgePulse)lastEdgePulse));
                //lastEdgePulse = new ProtocolBase.EdgePulse(i, 0, nowEdge);
                lastEdgePulse = new ProtocolBase.EdgePulse(i, 0, nowEdge);
            }
            //test point
            //if (i >= 117704)
            //{

            //}
        }
        return true;
    }
}
